@@ -1,5 +1,4 @@
 require 'test/unit'
-require_relative 'envutil'
 
 # to supress warnings for future calls of Module#refine
 EnvUtil.suppress_warning do
@@ -259,7 +258,7 @@ class TestRefinement < Test::Unit::TestCase
   def test_return_value_of_refine
     mod = nil
     result = nil
-    m = Module.new {
+    Module.new {
       result = refine(Object) {
         mod = self
       }
@@ -433,7 +432,6 @@ class TestRefinement < Test::Unit::TestCase
   end
 
   def test_module_using_class
-    c = Class.new
     assert_raise(TypeError) do
       eval("using TestRefinement::UsingClass", TOPLEVEL_BINDING)
     end
@@ -1219,6 +1217,24 @@ class TestRefinement < Test::Unit::TestCase
     end;
   end
 
+  def test_singleton_method_should_not_use_refinements
+    assert_separately([], <<-"end;")
+    bug10744 = '[ruby-core:67603] [Bug #10744]'
+
+    class C
+    end
+
+    module RefinementBug
+      refine C.singleton_class do
+        def foo
+        end
+      end
+    end
+
+    assert_raise(NameError, bug10744) { C.singleton_method(:foo) }
+    end;
+  end
+
   def test_refined_method_defined
     assert_separately([], <<-"end;")
     bug10753 = '[ruby-core:67656] [Bug #10753]'
