@@ -2,7 +2,7 @@
 
   vm_core.h -
 
-  $Author: charliesome $
+  $Author: naruse $
   created at: 04/01/01 19:41:38 JST
 
   Copyright (C) 2004-2007 Koichi Sasada
@@ -23,8 +23,14 @@
 #include "id.h"
 #include "method.h"
 #include "ruby_atomic.h"
+#include "ccan/list/list.h"
 
-#include "thread_native.h"
+#include "ruby/thread_native.h"
+#if   defined(_WIN32)
+#include "thread_win32.h"
+#elif defined(HAVE_PTHREAD_H)
+#include "thread_pthread.h"
+#endif
 
 #ifndef ENABLE_VM_OBJSPACE
 #ifdef _WIN32
@@ -100,27 +106,6 @@
 #endif /* OPT_STACK_CACHING */
 #endif /* OPT_CALL_THREADED_CODE */
 
-/* likely */
-#if __GNUC__ >= 3
-#define LIKELY(x)   (__builtin_expect((x), 1))
-#define UNLIKELY(x) (__builtin_expect((x), 0))
-#else /* __GNUC__ >= 3 */
-#define LIKELY(x)   (x)
-#define UNLIKELY(x) (x)
-#endif /* __GNUC__ >= 3 */
-
-#ifndef __has_attribute
-# define __has_attribute(x) 0
-#endif
-
-#if __has_attribute(unused)
-#define UNINITIALIZED_VAR(x) x __attribute__((unused))
-#elif defined(__GNUC__) && __GNUC__ >= 3
-#define UNINITIALIZED_VAR(x) x = x
-#else
-#define UNINITIALIZED_VAR(x) x
-#endif
-
 typedef unsigned long rb_num_t;
 
 /* iseq data type */
@@ -142,7 +127,6 @@ union iseq_inline_storage_entry {
     struct {
 	struct rb_thread_struct *running_thread;
 	VALUE value;
-	VALUE done;
     } once;
     struct iseq_inline_cache_entry cache;
 };
@@ -151,13 +135,20 @@ union iseq_inline_storage_entry {
 struct rb_thread_struct;
 struct rb_control_frame_struct;
 
+typedef struct rb_call_info_kw_arg_struct {
+    int keyword_len;
+    VALUE keywords[1];
+} rb_call_info_kw_arg_t;
+
 /* rb_call_info_t contains calling information including inline cache */
 typedef struct rb_call_info_struct {
     /* fixed at compile time */
     ID mid;
-    VALUE flag;
+
+    unsigned int flag;
     int orig_argc;
     rb_iseq_t *blockiseq;
+    rb_call_info_kw_arg_t *kw_arg;
 
     /* inline cache: keys */
     rb_serial_t method_state;
@@ -169,12 +160,12 @@ typedef struct rb_call_info_struct {
     VALUE defined_class;
 
     /* temporary values for method calling */
-    int argc;
     struct rb_block_struct *blockptr;
     VALUE recv;
+    int argc;
     union {
 	int opt_pc; /* used by iseq */
-	long index; /* used by ivar */
+	int index; /* used by ivar */
 	int missing_reason; /* used by method_missing */
 	int inc_sp; /* used by cfunc */
     } aux;
@@ -198,7 +189,7 @@ typedef struct rb_iseq_location_struct {
     const VALUE absolute_path;
     const VALUE base_label;
     const VALUE label;
-    size_t first_lineno;
+    VALUE first_lineno; /* TODO: may be unsigned short */
 } rb_iseq_location_t;
 
 struct rb_iseq_struct;
@@ -219,18 +210,22 @@ struct rb_iseq_struct {
 	ISEQ_TYPE_MAIN,
 	ISEQ_TYPE_DEFINED_GUARD
     } type;              /* instruction sequence type */
+#if defined(WORDS_BIGENDIAN) && (SIZEOF_VALUE > SIZEOF_INT)
+    char dummy[SIZEOF_VALUE - SIZEOF_INT]; /* [Bug #10037][ruby-core:63721] */
+#endif
+    int stack_max; /* for stack overflow check */
 
     rb_iseq_location_t location;
 
-    VALUE *iseq;         /* iseq (insn number and operands) */
-    VALUE *iseq_encoded; /* encoded iseq */
-    unsigned long iseq_size;
+    VALUE *iseq_encoded; /* encoded iseq (insn addr and operands) */
+    unsigned int iseq_size;
+    unsigned int line_info_size;
+
     const VALUE mark_ary;     /* Array: includes operands which should be GC marked */
     const VALUE coverage;     /* coverage array */
 
     /* insn info, must be freed */
     struct iseq_line_info_entry *line_info_table;
-    size_t line_info_size;
 
     ID *local_table;		/* must free */
     int local_table_size;
@@ -241,56 +236,81 @@ struct rb_iseq_struct {
     union iseq_inline_storage_entry *is_entries;
     int is_size;
 
-    rb_call_info_t *callinfo_entries;
     int callinfo_size;
+    rb_call_info_t *callinfo_entries;
 
     /**
-     * argument information
+     * parameter information
      *
      *  def m(a1, a2, ..., aM,                    # mandatory
      *        b1=(...), b2=(...), ..., bN=(...),  # optional
      *        *c,                                 # rest
      *        d1, d2, ..., dO,                    # post
      *        e1:(...), e2:(...), ..., eK:(...),  # keyword
-     *        **f,                                # keyword rest
+     *        **f,                                # keyword_rest
      *        &g)                                 # block
      * =>
      *
-     *  argc           = M                 // or  0 if no mandatory arg
-     *  arg_opts       = N+1               // or  0 if no optional arg
-     *  arg_rest       = M+N               // or -1 if no rest arg
-     *  arg_opt_table  = [ (arg_opts entries) ]
-     *  arg_post_start = M+N+(*1)          // or 0 if no post arguments
-     *  arg_post_len   = O                 // or 0 if no post arguments
-     *  arg_keywords   = K                 // or 0 if no keyword arg
-     *  arg_block      = M+N+(*1)+O+K      // or -1 if no block arg
-     *  arg_keyword    = M+N+(*1)+O+K+(&1) // or -1 if no keyword arg/rest
-     *  arg_simple     = 0 if not simple arguments.
-     *                 = 1 if no opt, rest, post, block.
-     *                 = 2 if ambiguous block parameter ({|a|}).
-     *  arg_size       = M+N+O+(*1)+K+(&1)+(**1) argument size.
+     *  lead_num     = M
+     *  opt_num      = N
+     *  rest_start   = M+N
+     *  post_start   = M+N+(*1)
+     *  post_num     = O
+     *  keyword_num  = K
+     *  block_start  = M+N+(*1)+O+K
+     *  keyword_bits = M+N+(*1)+O+K+(&1)
+     *  size         = M+N+O+(*1)+K+(&1)+(**1) // parameter size.
      */
 
-    int argc;
-    int arg_simple;
-    int arg_rest;
-    int arg_block;
-    int arg_opts;
-    int arg_post_len;
-    int arg_post_start;
-    int arg_size;
-    VALUE *arg_opt_table;
-    int arg_keyword;
-    int arg_keyword_check; /* if this is true, raise an ArgumentError when unknown keyword argument is passed */
-    int arg_keywords;
-    int arg_keyword_required;
-    ID *arg_keyword_table;
+    struct {
+	struct {
+	    unsigned int has_lead   : 1;
+	    unsigned int has_opt    : 1;
+	    unsigned int has_rest   : 1;
+	    unsigned int has_post   : 1;
+	    unsigned int has_kw     : 1;
+	    unsigned int has_kwrest : 1;
+	    unsigned int has_block  : 1;
 
-    size_t stack_max; /* for stack overflow check */
+	    unsigned int ambiguous_param0 : 1; /* {|a|} */
+	} flags;
+
+	int size;
+
+	int lead_num;
+	int opt_num;
+	int rest_start;
+	int post_start;
+	int post_num;
+	int block_start;
+
+	VALUE *opt_table; /* (opt_num + 1) entries. */
+	/* opt_num and opt_table:
+	 *
+	 * def foo o1=e1, o2=e2, ..., oN=eN
+	 * #=>
+	 *   # prologue code
+	 *   A1: e1
+	 *   A2: e2
+	 *   ...
+	 *   AN: eN
+	 *   AL: body
+	 * opt_num = N
+	 * opt_table = [A1, A2, ..., AN, AL]
+	 */
+
+	struct rb_iseq_param_keyword {
+	    int num;
+	    int required_num;
+	    int bits_start;
+	    int rest_start;
+	    ID *table;
+	    VALUE *default_values;
+	} *keyword;
+    } param;
 
     /* catch table */
-    struct iseq_catch_table_entry *catch_table;
-    int catch_table_size;
+    struct iseq_catch_table *catch_table;
 
     /* for child iseq */
     struct rb_iseq_struct *parent_iseq;
@@ -321,6 +341,10 @@ struct rb_iseq_struct {
 
     /* used at compile time */
     struct iseq_compile_data *compile_data;
+
+    /* original iseq, before encoding
+     * used for debug/dump (TODO: union with compile_data) */
+    VALUE *iseq;
 };
 
 enum ruby_special_exceptions {
@@ -331,6 +355,33 @@ enum ruby_special_exceptions {
     ruby_special_error_count
 };
 
+enum ruby_basic_operators {
+    BOP_PLUS,
+    BOP_MINUS,
+    BOP_MULT,
+    BOP_DIV,
+    BOP_MOD,
+    BOP_EQ,
+    BOP_EQQ,
+    BOP_LT,
+    BOP_LE,
+    BOP_LTLT,
+    BOP_AREF,
+    BOP_ASET,
+    BOP_LENGTH,
+    BOP_SIZE,
+    BOP_EMPTY_P,
+    BOP_SUCC,
+    BOP_GT,
+    BOP_GE,
+    BOP_NOT,
+    BOP_NEQ,
+    BOP_MATCH,
+    BOP_FREEZE,
+
+    BOP_LAST_
+};
+
 #define GetVMPtr(obj, ptr) \
   GetCoreDataFromValue((obj), rb_vm_t, (ptr))
 
@@ -354,7 +405,8 @@ typedef struct rb_vm_struct {
     struct rb_thread_struct *main_thread;
     struct rb_thread_struct *running_thread;
 
-    st_table *living_threads;
+    struct list_head living_threads;
+    size_t living_thread_num;
     VALUE thgroup_default;
 
     int running;
@@ -365,7 +417,7 @@ typedef struct rb_vm_struct {
     /* object management */
     VALUE mark_object_ary;
 
-    VALUE special_exceptions[ruby_special_error_count];
+    const VALUE special_exceptions[ruby_special_error_count];
 
     /* load */
     VALUE top_self;
@@ -414,6 +466,7 @@ typedef struct rb_vm_struct {
     struct RArray at_exit;
 
     VALUE *defined_strings;
+    st_table *frozen_strings;
 
     /* params */
     struct { /* size in byte */
@@ -422,6 +475,8 @@ typedef struct rb_vm_struct {
 	size_t fiber_vm_stack_size;
 	size_t fiber_machine_stack_size;
     } default_params;
+
+    short redefined_flag[BOP_LAST_];
 } rb_vm_t;
 
 /* default values */
@@ -438,10 +493,27 @@ typedef struct rb_vm_struct {
 #define RUBY_VM_FIBER_MACHINE_STACK_SIZE      (  64 * 1024 * sizeof(VALUE)) /*  256 KB or  512 KB */
 #define RUBY_VM_FIBER_MACHINE_STACK_SIZE_MIN  (  16 * 1024 * sizeof(VALUE)) /*   64 KB or  128 KB */
 
+/* optimize insn */
+#define FIXNUM_REDEFINED_OP_FLAG (1 << 0)
+#define FLOAT_REDEFINED_OP_FLAG  (1 << 1)
+#define STRING_REDEFINED_OP_FLAG (1 << 2)
+#define ARRAY_REDEFINED_OP_FLAG  (1 << 3)
+#define HASH_REDEFINED_OP_FLAG   (1 << 4)
+#define BIGNUM_REDEFINED_OP_FLAG (1 << 5)
+#define SYMBOL_REDEFINED_OP_FLAG (1 << 6)
+#define TIME_REDEFINED_OP_FLAG   (1 << 7)
+#define REGEXP_REDEFINED_OP_FLAG (1 << 8)
+
+#define BASIC_OP_UNREDEFINED_P(op, klass) (LIKELY((GET_VM()->redefined_flag[(op)]&(klass)) == 0))
+
 #ifndef VM_DEBUG_BP_CHECK
 #define VM_DEBUG_BP_CHECK 0
 #endif
 
+#ifndef VM_DEBUG_VERIFY_METHOD_CACHE
+#define VM_DEBUG_VERIFY_METHOD_CACHE 0
+#endif
+
 typedef struct rb_control_frame_struct {
     VALUE *pc;			/* cfp[0] */
     VALUE *sp;			/* cfp[1] */
@@ -521,7 +593,12 @@ typedef struct rb_ensure_list {
     struct rb_ensure_entry entry;
 } rb_ensure_list_t;
 
+typedef char rb_thread_id_string_t[sizeof(rb_nativethread_id_t) * 2 + 3];
+
+typedef struct rb_fiber_struct rb_fiber_t;
+
 typedef struct rb_thread_struct {
+    struct list_node vmlt_node;
     VALUE self;
     rb_vm_t *vm;
 
@@ -542,7 +619,7 @@ typedef struct rb_thread_struct {
     const rb_block_t *passed_block;
 
     /* for bmethod */
-    const rb_method_entry_t *passed_me;
+    const rb_method_entry_t *passed_bmethod_me;
 
     /* for cfunc */
     rb_call_info_t *passed_ci;
@@ -559,9 +636,13 @@ typedef struct rb_thread_struct {
 
     /* thread control */
     rb_nativethread_id_t thread_id;
+#ifdef NON_SCALAR_THREAD_ID
+    rb_thread_id_string_t thread_id_string;
+#endif
     enum rb_thread_status status;
     int to_kill;
     int priority;
+    int mark_stack_len;
 
     native_thread_data_t native_thread_data;
     void *blocking_region_buffer;
@@ -579,8 +660,8 @@ typedef struct rb_thread_struct {
 
     /* async errinfo queue */
     VALUE pending_interrupt_queue;
-    int pending_interrupt_queue_checked;
     VALUE pending_interrupt_mask_stack;
+    int pending_interrupt_queue_checked;
 
     rb_atomic_t interrupt_flag;
     unsigned long interrupt_mask;
@@ -609,6 +690,8 @@ typedef struct rb_thread_struct {
 
     /* storage */
     st_table *local_storage;
+    VALUE local_storage_recursive_hash;
+    VALUE local_storage_recursive_hash_for_trace;
 
     rb_thread_list_t *join_list;
 
@@ -617,16 +700,17 @@ typedef struct rb_thread_struct {
     VALUE (*first_func)(ANYARGS);
 
     /* for GC */
-    VALUE *machine_stack_start;
-    VALUE *machine_stack_end;
-    size_t machine_stack_maxsize;
+    struct {
+	VALUE *stack_start;
+	VALUE *stack_end;
+	size_t stack_maxsize;
 #ifdef __ia64
-    VALUE *machine_register_stack_start;
-    VALUE *machine_register_stack_end;
-    size_t machine_register_stack_maxsize;
+	VALUE *register_stack_start;
+	VALUE *register_stack_end;
+	size_t register_stack_maxsize;
 #endif
-    jmp_buf machine_regs;
-    int mark_stack_len;
+	jmp_buf regs;
+    } machine;
 
     /* statistics data for profiler */
     VALUE stat_insn_usage;
@@ -636,8 +720,8 @@ typedef struct rb_thread_struct {
     struct rb_trace_arg_struct *trace_arg; /* trace information */
 
     /* fiber */
-    VALUE fiber;
-    VALUE root_fiber;
+    rb_fiber_t *fiber;
+    rb_fiber_t *root_fiber;
     rb_jmpbuf_t root_jmpbuf;
 
     /* ensure & callcc */
@@ -683,7 +767,7 @@ VALUE rb_iseq_compile_on_base(VALUE src, VALUE file, VALUE line, rb_block_t *bas
 VALUE rb_iseq_compile_with_option(VALUE src, VALUE file, VALUE absolute_path, VALUE line, rb_block_t *base_block, VALUE opt);
 
 VALUE rb_iseq_disasm(VALUE self);
-int rb_iseq_disasm_insn(VALUE str, VALUE *iseqval, size_t pos, rb_iseq_t *iseq, VALUE child);
+int rb_iseq_disasm_insn(VALUE str, const VALUE *iseqval, size_t pos, const rb_iseq_t *iseq, VALUE child);
 const char *ruby_node_name(int node);
 
 RUBY_EXTERN VALUE rb_cISeq;
@@ -700,20 +784,20 @@ typedef struct {
 
     VALUE envval;		/* for GC mark */
     VALUE blockprocval;
-    int safe_level;
-    int is_from_method;
-    int is_lambda;
+    int8_t safe_level;		/* 0..3 */
+    int8_t is_from_method;	/* bool */
+    int8_t is_lambda;		/* bool */
 } rb_proc_t;
 
 #define GetEnvPtr(obj, ptr) \
   GetCoreDataFromValue((obj), rb_env_t, (ptr))
 
 typedef struct {
-    VALUE *env;
     int env_size;
     int local_size;
     VALUE prev_envval;		/* for GC mark */
     rb_block_t block;
+    VALUE env[1];               /* flexible array */
 } rb_env_t;
 
 extern const rb_data_type_t ruby_binding_data_type;
@@ -724,6 +808,7 @@ extern const rb_data_type_t ruby_binding_data_type;
 typedef struct {
     VALUE env;
     VALUE path;
+    VALUE blockprocval;	/* for GC mark */
     unsigned short first_lineno;
 } rb_binding_t;
 
@@ -745,7 +830,7 @@ enum vm_check_match_type {
 #define VM_CALL_TAILCALL        (0x01 << 5) /* located at tail position */
 #define VM_CALL_SUPER           (0x01 << 6) /* super */
 #define VM_CALL_OPT_SEND        (0x01 << 7) /* internal flag */
-#define VM_CALL_ARGS_SKIP_SETUP (0x01 << 8) /* (flag & (SPLAT|BLOCKARG)) && blockiseq == 0 */
+#define VM_CALL_ARGS_SIMPLE     (0x01 << 8) /* (ci->flag & (SPLAT|BLOCKARG)) && ci->blockiseq == NULL && ci->kw_arg == NULL */
 
 enum vm_special_object_type {
     VM_SPECIAL_OBJECT_VMCORE = 1,
@@ -762,15 +847,18 @@ enum vm_special_object_type {
 #define VM_FRAME_MAGIC_IFUNC  0x81
 #define VM_FRAME_MAGIC_EVAL   0x91
 #define VM_FRAME_MAGIC_LAMBDA 0xa1
-#define VM_FRAME_MAGIC_MASK_BITS   8
-#define VM_FRAME_MAGIC_MASK   (~(~0<<VM_FRAME_MAGIC_MASK_BITS))
+#define VM_FRAME_MAGIC_RESCUE 0xb1
+#define VM_FRAME_MAGIC_MASK_BITS 8
+#define VM_FRAME_MAGIC_MASK   (~(~(VALUE)0<<VM_FRAME_MAGIC_MASK_BITS))
 
 #define VM_FRAME_TYPE(cfp) ((cfp)->flag & VM_FRAME_MAGIC_MASK)
 
 /* other frame flag */
-#define VM_FRAME_FLAG_PASSED 0x0100
-#define VM_FRAME_FLAG_FINISH 0x0200
-#define VM_FRAME_TYPE_FINISH_P(cfp) (((cfp)->flag & VM_FRAME_FLAG_FINISH) != 0)
+#define VM_FRAME_FLAG_PASSED  0x0100
+#define VM_FRAME_FLAG_FINISH  0x0200
+#define VM_FRAME_FLAG_BMETHOD 0x0400
+#define VM_FRAME_TYPE_FINISH_P(cfp)  (((cfp)->flag & VM_FRAME_FLAG_FINISH) != 0)
+#define VM_FRAME_TYPE_BMETHOD_P(cfp) (((cfp)->flag & VM_FRAME_FLAG_BMETHOD) != 0)
 
 #define RUBYVM_CFUNC_FRAME_P(cfp) \
   (VM_FRAME_TYPE(cfp) == VM_FRAME_MAGIC_CFUNC)
@@ -836,7 +924,8 @@ rb_block_t *rb_vm_control_frame_block_ptr(rb_control_frame_t *cfp);
 
 /* VM related object allocate functions */
 VALUE rb_thread_alloc(VALUE klass);
-VALUE rb_proc_alloc(VALUE klass);
+VALUE rb_proc_wrap(VALUE klass, rb_proc_t *); /* may use with rb_proc_alloc */
+VALUE rb_binding_alloc(VALUE klass);
 
 /* for debug */
 extern void rb_vmdebug_stack_dump_raw(rb_thread_t *, rb_control_frame_t *);
@@ -845,7 +934,8 @@ extern void rb_vmdebug_debug_print_post(rb_thread_t *th, rb_control_frame_t *cfp
 
 #define SDR() rb_vmdebug_stack_dump_raw(GET_THREAD(), GET_THREAD()->cfp)
 #define SDR2(cfp) rb_vmdebug_stack_dump_raw(GET_THREAD(), (cfp))
-void rb_vm_bugreport(void);
+void rb_vm_bugreport(const void *);
+NORETURN(void rb_bug_context(const void *, const char *fmt, ...));
 
 /* functions about thread/vm execution */
 RUBY_SYMBOL_EXPORT_BEGIN
@@ -856,9 +946,11 @@ int rb_thread_method_id_and_class(rb_thread_t *th, ID *idp, VALUE *klassp);
 
 VALUE rb_vm_invoke_proc(rb_thread_t *th, rb_proc_t *proc,
 			int argc, const VALUE *argv, const rb_block_t *blockptr);
+VALUE rb_vm_make_proc_lambda(rb_thread_t *th, const rb_block_t *block, VALUE klass, int8_t is_lambda);
 VALUE rb_vm_make_proc(rb_thread_t *th, const rb_block_t *block, VALUE klass);
+VALUE rb_vm_make_binding(rb_thread_t *th, const rb_control_frame_t *src_cfp);
 VALUE rb_vm_make_env_object(rb_thread_t *th, rb_control_frame_t *cfp);
-VALUE rb_binding_new_with_cfp(rb_thread_t *th, const rb_control_frame_t *src_cfp);
+VALUE rb_vm_env_local_variables(VALUE envval);
 VALUE *rb_binding_add_dynavars(rb_binding_t *bind, int dyncount, const ID *dynvars);
 void rb_vm_inc_const_missing_count(void);
 void rb_vm_gvl_destroy(rb_vm_t *vm);
@@ -873,20 +965,46 @@ void rb_thread_stop_timer_thread(int);
 void rb_thread_reset_timer_thread(void);
 void rb_thread_wakeup_timer_thread(void);
 
+static inline void
+rb_vm_living_threads_init(rb_vm_t *vm)
+{
+    list_head_init(&vm->living_threads);
+    vm->living_thread_num = 0;
+}
+
+static inline void
+rb_vm_living_threads_insert(rb_vm_t *vm, rb_thread_t *th)
+{
+    list_add_tail(&vm->living_threads, &th->vmlt_node);
+    vm->living_thread_num++;
+}
+
+static inline void
+rb_vm_living_threads_remove(rb_vm_t *vm, rb_thread_t *th)
+{
+    list_del(&th->vmlt_node);
+    vm->living_thread_num--;
+}
+
 int ruby_thread_has_gvl_p(void);
 typedef int rb_backtrace_iter_func(void *, VALUE, int, VALUE);
-rb_control_frame_t *rb_vm_get_ruby_level_next_cfp(rb_thread_t *th, const rb_control_frame_t *cfp);
-rb_control_frame_t *rb_vm_get_binding_creatable_next_cfp(rb_thread_t *th, const rb_control_frame_t *cfp);
+rb_control_frame_t *rb_vm_get_ruby_level_next_cfp(const rb_thread_t *th, const rb_control_frame_t *cfp);
+rb_control_frame_t *rb_vm_get_binding_creatable_next_cfp(const rb_thread_t *th, const rb_control_frame_t *cfp);
 int rb_vm_get_sourceline(const rb_control_frame_t *);
 VALUE rb_name_err_mesg_new(VALUE obj, VALUE mesg, VALUE recv, VALUE method);
 void rb_vm_stack_to_heap(rb_thread_t *th);
 void ruby_thread_init_stack(rb_thread_t *th);
 int rb_vm_control_frame_id_and_class(const rb_control_frame_t *cfp, ID *idp, VALUE *klassp);
+void rb_vm_rewind_cfp(rb_thread_t *th, rb_control_frame_t *cfp);
+
+void rb_vm_register_special_exception(enum ruby_special_exceptions sp, VALUE exception_class, const char *mesg);
 
 void rb_gc_mark_machine_stack(rb_thread_t *th);
 
 int rb_autoloading_value(VALUE mod, ID id, VALUE* value);
 
+void rb_vm_rewrite_cref_stack(NODE *node, VALUE old_klass, VALUE new_klass, NODE **new_cref_ptr);
+
 #define sysstack_error GET_VM()->special_exceptions[ruby_error_sysstack]
 
 #define RUBY_CONST_ASSERT(expr) (1/!!(expr)) /* expr must be a compile-time constant */
